// Simmulator Inputs (coils)
#define RIVER_VALVE		0
#define TANK2_INVALVE	1
#define TANK2_OUTVALVE	2

// Simmulator Outputs (input registers)
#define FUEL_VALVE		0
#define STEAM_VALVE		1
#define WATER_VALVE		2

#define OUTPUT_INDEX		12

double g_setpts[ 41 ];

#include <stdio.h>
extern "C"
{

void initialize(unsigned char* di, unsigned char* coil, unsigned short* ir, unsigned short* hr, double* genr)
{
	for ( int i = 0 ; i < 40 ; ++i )
	{
		g_setpts[ i ] = 0;
	}
	
	g_setpts[ 1 ]  = 3678.549276;
	g_setpts[ 39 ] = 1.225791;
	g_setpts[ 40 ] = 0.816486;
	
	genr[ 0 ] = 63.053;
}


static int g_counter = 0;
void execStep(unsigned char* di, unsigned char* coil, unsigned short* ir, unsigned short* hr, double* genr)
{
	++g_counter;
	if ( g_counter >= 138 )
	{
		// Calculate setpoint
		static double prev_err_s = 0;
		const double Kc_s = 0.1;
		const double Ts_s = 0.1; //72micro - the time step
		const double Ti_s = 50*60; //seconds
		double err = g_setpts[ 39 ] - genr[ OUTPUT_INDEX + 39 ] / genr[ OUTPUT_INDEX + 40 ];

		g_setpts[ 40 ] = g_setpts[ 40 ] + Kc_s * ( err - prev_err_s + (Ts_s / Ti_s)*err );
		g_setpts[ 1 ] = g_setpts[ 40 ] * genr[ OUTPUT_INDEX + 2 ];
		prev_err_s = err;

		// Calculate new valve position
		static double prev_err_v = 0;
		const double Kc_v = 0.0115;
		const double Ts_v = 0.1; //72micro - the time step
		const double Ti_v = 6; //seconds: 60s/0.1=6s
		double err1 = g_setpts[ 1 ] - genr[ OUTPUT_INDEX + 1 ];
		genr[ 0 ] = genr[ 0 ] + Kc_v * ( err1 - prev_err_v + (Ts_v / Ti_v)*err1 );
		prev_err_v = err1;
		
		g_counter = 0;
		
		if ( genr[ 0 ] > 100 ) {
			genr[ 0 ] = 100;
		}
		if ( genr[ 0 ] < 0 ) {
			genr[ 0 ] = 0;
		}
	}
}

void getRequiredReadRegisters( unsigned char* di, unsigned char* coil, unsigned char* ir, unsigned char* hr, unsigned char* genr )
{
}

void getRequiredWriteRegisters( unsigned char* di, unsigned char* coil, unsigned char* ir, unsigned char* hr, unsigned char* genr )
{
}


} //extern "C"
