// Simmulator Inputs (coils)
#define RIVER_VALVE		0
#define TANK2_INVALVE	1
#define TANK2_OUTVALVE	2

// Simmulator Outputs (input registers)
#define FUEL_VALVE		0
#define STEAM_VALVE		1
#define WATER_VALVE		2

#define OUTPUT_INDEX		12

double g_setpts[ 41 ];


#include <stdio.h>
extern "C"
{

void initialize(unsigned char* di, unsigned char* coil, unsigned short* ir, unsigned short* hr, double* genr)
{
	for ( int i = 0 ; i < 40 ; ++i )
	{
		g_setpts[ i ] = 0;
	}
	
	g_setpts[ 4 ]  = 26.926537;
	g_setpts[ 21 ] = 77.312367;
	
	genr[ 10 ] = 18.114;
	genr[ 11 ] = 50; // no associated control loop
}

static int g_counter = 0;
void execStep(unsigned char* di, unsigned char* coil, unsigned short* ir, unsigned short* hr, double* genr)
{
	++g_counter;
	if ( g_counter > 138 )
	{
		// Calculate setpoint
		static double prev_err_s = 0;
		const double Kc_s = 1;
		const double Ts_s = 0.1; //72micro - the time step
		const double Ti_s = 25*60; //seconds
		double err = g_setpts[ 4 ] - genr[ OUTPUT_INDEX + 4 ];
		g_setpts[ 21 ] = g_setpts[ 21 ] + Kc_s * ( err - prev_err_s + (Ts_s / Ti_s)*err );
		prev_err_s = err;

		// Calculate new valve position
		static double prev_err_v = 0;
		const double Kc_v = -5;
		const double Ts_v = 0.1; //72micro - the time step
		const double Ti_v = 30; //seconds: 60s/0.5=30s
		double err1 = g_setpts[ 21 ] - genr[ OUTPUT_INDEX + 21 ];
		genr[ 10 ] = genr[ 10 ] + Kc_v * ( err1 - prev_err_v + (Ts_v / Ti_v)*err1 );
		prev_err_v = err1;
		
		g_counter = 0;
		
		if ( genr[ 10 ] > 100 ) {
			genr[ 10 ] = 100;
		}
		if ( genr[ 10 ] < 0 ) {
			genr[ 10 ] = 0;
		}

	}
}

void getRequiredReadRegisters( unsigned char* di, unsigned char* coil, unsigned char* ir, unsigned char* hr, unsigned char* genr )
{
}

void getRequiredWriteRegisters( unsigned char* di, unsigned char* coil, unsigned char* ir, unsigned char* hr, unsigned char* genr )
{
}


} //extern "C"
