/*
* Copyright 2013 EUROPEAN UNION
*
* Licenced under the EUPL, Version 1.1 only (the "Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* http://ec.europa.eu/idabc/eupl.html
*
* Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and limitations under the Licence.
*/

#include "DynaLoader.h"
#include <stdio.h>

typedef struct
{
    DynaLoader* pldr;
	void (*fptr_setPlcName)( char* );
    void (*fptr_initialize)( unsigned char*, unsigned char*, unsigned short*, unsigned short*, double* );
    void (*fptr_execStep)( unsigned char*, unsigned char*, unsigned short*, unsigned short*, double* );
    void (*fptr_getRequiredReadRegisters)( unsigned char*, unsigned char*, unsigned char*, unsigned char*, unsigned char* );
    void (*fptr_getRequiredWriteRegisters)( unsigned char*, unsigned char*, unsigned char*, unsigned char*, unsigned char* );
}PLC_FPTR;
#include <errno.h>
extern "C"
{
bool setPlcName( int ptr, char* name );
int loadLibrary( char* pName )
{
    DynaLoader* pldr = new DynaLoader(pName);
    if ( !pldr->loadModule() )
    {
	delete pldr;
	return -1;
    }
    
    PLC_FPTR* ps = new PLC_FPTR;
	ps->fptr_setPlcName = reinterpret_cast< void (*)(char*) >( pldr->getFunction( "setPlcName" ) );
    if ( NULL == ps->fptr_setPlcName )
    {
	delete ps;
	delete pldr;
	return -1;
    }
	
	setPlcName( reinterpret_cast< int >( ps ), pName );
	
    ps->fptr_initialize = reinterpret_cast< void (*)(unsigned char*, unsigned char*, unsigned short*, unsigned short*, double*) >( pldr->getFunction( "initialize" ) );
    if ( NULL == ps->fptr_initialize )
    {
	delete ps;
	delete pldr;
	return -1;
    }

    ps->fptr_execStep = reinterpret_cast< void (*)(unsigned char*, unsigned char*, unsigned short*, unsigned short*, double*) >( pldr->getFunction( "execStep" ) );
    if ( NULL == ps->fptr_execStep )
    {
	delete ps;
	delete pldr;
	return -1;
    }
    
    
    ps->fptr_getRequiredReadRegisters = reinterpret_cast< void (*)(unsigned char*, unsigned char*, unsigned char*, unsigned char*, unsigned char*) >( pldr->getFunction( "getRequiredReadRegisters" ) );
    if ( NULL == ps->fptr_execStep )
    {
	delete ps;
	delete pldr;
	return -1;
    }

    ps->fptr_getRequiredWriteRegisters = reinterpret_cast< void (*)(unsigned char*, unsigned char*, unsigned char*, unsigned char*, unsigned char*) >( pldr->getFunction( "getRequiredWriteRegisters" ) );
    if ( NULL == ps->fptr_getRequiredWriteRegisters )
    {
	delete ps;
	delete pldr;
	return -1;
    }

    ps->pldr = pldr;
    
    return ( reinterpret_cast< int >( ps ) );
}

void cleanup( long ptr )
{
    if ( ptr <= 0 )
    {
	return;
    }
    
    PLC_FPTR* ps = reinterpret_cast< PLC_FPTR* >( ptr );
    delete ps->pldr;
    delete ps;
}

bool setPlcName( int ptr, char* name )
{
    reinterpret_cast< PLC_FPTR* >( ptr )->fptr_setPlcName( name );
    return true;
}

bool initialize( int ptr, unsigned char* di, unsigned char* coil, unsigned short* ir, unsigned short* hr, double* genr )
{
    reinterpret_cast< PLC_FPTR* >( ptr )->fptr_initialize( di, coil, ir, hr, genr );
    return true;
}

bool getRequiredReadRegisters( int ptr, unsigned char* di, unsigned char* coil, unsigned char* ir, unsigned char* hr, unsigned char* genr )
{
    reinterpret_cast< PLC_FPTR* >( ptr )->fptr_getRequiredReadRegisters( di, coil, ir, hr, genr );
    return true;
}

bool getRequiredWriteRegisters( int ptr, unsigned char* di, unsigned char* coil, unsigned char* ir, unsigned char* hr, unsigned char* genr )
{
    reinterpret_cast< PLC_FPTR* >( ptr )->fptr_getRequiredWriteRegisters( di, coil, ir, hr, genr );
    return true;
}

bool execStep( int ptr, unsigned char* di, unsigned char* coil, unsigned short* ir, unsigned short* hr, double* genr )
{
    reinterpret_cast< PLC_FPTR* >( ptr )->fptr_execStep( di, coil, ir, hr, genr );
    return true;
}

} // extern "C"
